=== modified file 'bzrlib/tests/test_merge_directive.py'
--- bzrlib/tests/test_merge_directive.py	2007-03-09 20:01:41 +0000
+++ bzrlib/tests/test_merge_directive.py	2007-03-09 20:41:48 +0000
@@ -23,9 +23,29 @@
     )
 
 
+OUTPUT1 = """# Bazaar merge directive format experimental-1
+# revision_id: example:
+# target_branch: http://example.com
+# testament_sha1: sha
+# timestamp: 1970-01-01 00:09:33 +0002
+#\x20
+booga"""
+
+
+OUTPUT2 = """# Bazaar merge directive format experimental-1
+# revision_id: example:
+# target_branch: http://example.com
+# testament_sha1: sha
+# timestamp: 1970-01-01 00:09:33 +0002
+# source_branch: http://example.org
+# message: Hi mom!
+#\x20
+booga"""
+
+
 class TestMergeDirective(tests.TestCase):
 
-    def test_init(self):
+    def test_merge_source(self):
         time = 500.0
         timezone = 5
         self.assertRaises(errors.NoMergeSource, merge_directive.MergeDirective,
@@ -33,24 +53,42 @@
         self.assertRaises(errors.NoMergeSource, merge_directive.MergeDirective,
             'example:', 'sha', time, timezone, 'http://example.com',
             patch_type='diff')
-        md = merge_directive.MergeDirective('example:', 'sha', time, timezone,
+        merge_directive.MergeDirective('example:', 'sha', time, timezone,
             'http://example.com', source_branch='http://example.org')
-        self.assertRaises(errors.PatchMissing, merge_directive.MergeDirective,
-            'example:', 'sha', time, timezone, 'http://example.com',
-            patch_type='bundle')
         md = merge_directive.MergeDirective('null:', 'sha', time, timezone,
             'http://example.com', patch='blah', patch_type='bundle')
+        self.assertIs(None, md.source_branch)
+        md2 = merge_directive.MergeDirective('null:', 'sha', time, timezone,
+            'http://example.com', patch='blah', patch_type='bundle',
+            source_branch='bar')
+        self.assertEqual('bar', md2.source_branch)
+
+    def test_require_patch(self):
+        time = 500.0
+        timezone = 5
         self.assertRaises(errors.PatchMissing, merge_directive.MergeDirective,
             'example:', 'sha', time, timezone, 'http://example.com',
-            source_branch="http://example.org", patch_type='diff')
+            patch_type='bundle')
         md = merge_directive.MergeDirective('example:', 'sha1', time, timezone,
             'http://example.com', source_branch="http://example.org",
             patch='', patch_type='diff')
+        self.assertEqual(md.patch, '')
 
     def test_serialization(self):
         time = 501
         timezone = 72
         md = merge_directive.MergeDirective('example:', 'sha', time, timezone,
+            'http://example.com', patch='booga', patch_type='bundle')
+        self.assertEqualDiff(OUTPUT1, ''.join(md.to_lines()))
+        md = merge_directive.MergeDirective('example:', 'sha', time, timezone,
+            'http://example.com', source_branch="http://example.org",
+            patch='booga', patch_type='diff', message="Hi mom!")
+        self.assertEqualDiff(OUTPUT2, ''.join(md.to_lines()))
+
+    def test_roundtrip(self):
+        time = 501
+        timezone = 72
+        md = merge_directive.MergeDirective('example:', 'sha', time, timezone,
             'http://example.com', source_branch="http://example.org",
             patch='booga', patch_type='diff')
         md2 = merge_directive.MergeDirective.from_lines(md.to_lines())
@@ -116,7 +154,17 @@
         tree_a.commit('Commit of rev2a', rev_id='rev2a')
         return tree_a, tree_b, branch_c
 
-    def test_generate(self):
+    def test_generate_patch(self):
+        tree_a, tree_b, branch_c = self.make_trees()
+        md2 = merge_directive.MergeDirective.from_objects(
+            tree_a.branch.repository, 'rev2a', 500, 144, tree_b.branch.base,
+            patch_type='diff', public_branch=tree_a.branch.base)
+        self.assertNotContainsRe(md2.patch, 'Bazaar revision bundle')
+        self.assertContainsRe(md2.patch, '\\+content_c')
+        self.assertNotContainsRe(md2.patch, '\\+\\+\\+ b/')
+        self.assertContainsRe(md2.patch, '\\+\\+\\+ file')
+
+    def test_public_branch(self):
         tree_a, tree_b, branch_c = self.make_trees()
         self.assertRaises(errors.PublicBranchOutOfDate,
             merge_directive.MergeDirective.from_objects,
@@ -126,19 +174,17 @@
         md1 = merge_directive.MergeDirective.from_objects(
             tree_a.branch.repository, 'rev2a', 500, 144, tree_b.branch.base,
             public_branch=branch_c.base)
-        self.assertContainsRe(md1.patch, 'Bazaar revision bundle')
-        self.assertContainsRe(md1.patch, '\\+content_c')
-        self.assertNotContainsRe(md1.patch, '\\+content_a')
         # public branch is provided with a bundle, despite possibly being out
         # of date, because it's not required if a bundle is present.
         self.assertEqual(md1.source_branch, branch_c.base)
+        # Once we update the public branch, we can generate a diff.
         branch_c.pull(tree_a.branch)
-        md2 = merge_directive.MergeDirective.from_objects(
+        md3 = merge_directive.MergeDirective.from_objects(
             tree_a.branch.repository, 'rev2a', 500, 144, tree_b.branch.base,
-            patch_type='diff', public_branch=branch_c.base)
-        self.assertNotContainsRe(md2.patch, 'Bazaar revision bundle')
-        self.assertContainsRe(md1.patch, '\\+content_c')
-        self.assertNotContainsRe(md1.patch, '\\+content_a')
+            patch_type=None, public_branch=branch_c.base)
+
+    def test_message(self):
+        tree_a, tree_b, branch_c = self.make_trees()
         md3 = merge_directive.MergeDirective.from_objects(
             tree_a.branch.repository, 'rev2a', 500, 144, tree_b.branch.base,
             patch_type=None, public_branch=branch_c.base,
@@ -147,15 +193,16 @@
         self.assertIs(None, md3.patch)
         self.assertEqual('Merge message', md3.message)
 
-    def test_generate_patch(self):
+    def test_generate_bundle(self):
         tree_a, tree_b, branch_c = self.make_trees()
-        md2 = merge_directive.MergeDirective.from_objects(
+        md1 = merge_directive.MergeDirective.from_objects(
             tree_a.branch.repository, 'rev2a', 500, 144, tree_b.branch.base,
-            patch_type='diff', public_branch=tree_a.branch.base)
-        self.assertNotContainsRe(md2.patch, 'Bazaar revision bundle')
-        self.assertContainsRe(md2.patch, '\\+content_c')
-        self.assertNotContainsRe(md2.patch, '\\+\\+\\+ b/')
-        self.assertContainsRe(md2.patch, '\\+\\+\\+ file')
+            public_branch=branch_c.base)
+        self.assertContainsRe(md1.patch, 'Bazaar revision bundle')
+        self.assertContainsRe(md1.patch, '\\+content_c')
+        self.assertNotContainsRe(md1.patch, '\\+content_a')
+        self.assertContainsRe(md1.patch, '\\+content_c')
+        self.assertNotContainsRe(md1.patch, '\\+content_a')
 
     def test_signing(self):
         time = 501

